import logging
from confluent_kafka import Consumer
from typing import Callable
from ..interfaces.ievent import IEvent 
from ..interfaces.iconsumer import  IConsumer

logger = logging.getLogger()

class KafkaConsumer(IConsumer):
    def __init__(self, broker: str,consumer_group: str):
        self.consumer = Consumer({
            'bootstrap.servers': broker,
            'group.id': consumer_group,
            'auto.offset.reset': 'earliest'
        })

    def subscribe(self, topic: str,  callback: Callable[[IEvent], None]) -> None:
        self.consumer.subscribe(topics=[topic])
                    
        while True:
            message = self.consumer.poll(1.0)
            if message is None:
                continue
            if message.error():
                logger.error(
                    f'Error occurred while consuming from Kafka: {message.error().str()}')
                continue
            callback(message.value)
            logger.info(f"Received Message on Common django: {message.value.decode('utf-8')}")

            
        


# middleware.py
import logging
from django.http import JsonResponse, HttpResponseServerError

logger = logging.getLogger()




class HealthCheckMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        if request.method == "GET":
            if request.path == "/health/ready":
                return self.readiness(request)
            elif request.path == "/health/live":
                return self.liveness(request)
        return self.get_response(request)

    def liveness(self, request):
        """
        Returns that the server is alive.
        """
        logger.info('Checking the liveness of the application')
        return JsonResponse({"message": "Application is live."}, status=200)

    def readiness(self, request):
        # Connect to each database and do a generic standard SQL query
        # that doesn't write any data and doesn't depend on any tables
        # being present.
        logger.info('Checking the readiness of the application')
        try:
            from django.db import connections
            for name in connections:
                cursor = connections[name].cursor()
                cursor.execute("SELECT 1;")
                row = cursor.fetchone()
                if row is None:
                    return HttpResponseServerError("db: invalid response")
        except Exception as e:
            logger.error(str(e))
            return HttpResponseServerError("db: cannot connect to database.")

        return JsonResponse({"message": "Application is ready."}, status=200)



from .iconsumer_configuration import IConsumerConfiguration
from .iconsumer import IConsumer
from .ievent import IEvent
from .imetadata import IMetadata
from .iproducer_configuration import IProducerConfiguration
from .iproducer import IProducer


from abc import ABC, abstractmethod
from .ievent import IEvent
from .imetadata import IMetadata

class IProducer(ABC):
    @abstractmethod
    def send_message(self, metadata: IMetadata, event: IEvent) -> None:
        pass



from abc import ABC, abstractmethod
from typing import Callable
from .ievent import IEvent

class IConsumer(ABC):
    @abstractmethod
    def subscribe(self, topic: str, callback: Callable[[IEvent], None]) -> None:
        pass



import json
import logging
from confluent_kafka import Producer
from ..interfaces.ievent import IEvent 
from ..interfaces.imetadata import IMetadata 
from ..interfaces.iproducer import IProducer 

logger = logging.getLogger()


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class MyMeta(type(IProducer), type(Singleton)):
    pass
class KafkaProducer(IProducer,metaclass=MyMeta):
    def __init__(self, broker: str):
        self.producer = Producer({'bootstrap.servers': broker})

    def acked(self, err, msg):
        if err is not None:
            logger.error(f'Failed to deliver message: {msg.value()}: {err.str()}')
        else:
            logger.info(f'Message produced: {msg.value()}')

    def send_message(self, metadata: IMetadata, message: IEvent) -> None:
        topic = metadata.topic
        serialized_message = json.dumps(message.__dict__())

        try:
            self.producer.produce(topic, serialized_message.encode('utf-8'), callback=self.acked)
            logger.info(f"Message sent to Kafka from django: {serialized_message}")
            self.producer.flush()
        except Exception as e:
            logger.error(f"Error occurred while producing to Kafka: {e}")
    
    @staticmethod
    def get_instance(broker: str) -> IProducer:
        return KafkaProducer(broker)



import json
from abc import ABC, abstractmethod
from typing import List, Optional


class IErrorStruct:
    def __init__(self, message: str, field: Optional[str] = None):
        self.message = message
        self.field = field

    def toJson(self):
        return json.dumps(self, default=lambda o: o.__dict__)


class CustomError(Exception, ABC):
    def __init__(self, message: str):
        super().__init__(message)

    @property
    @abstractmethod
    def status_code(self) -> int:
        pass

    @abstractmethod
    def serialize_errors(self) -> List[IErrorStruct]:
        pass



class IEvent:
    def __init__(self, eventType: str, data: any, createdAt: str):
        self.eventType = eventType
        self.data = data
        self.createdAt = createdAt
        
    def __dict__(self):
        return {
            'eventType': self.eventType,
            'data': self.data,
            'createdAt': self.createdAt
        }


from typing import Optional, List
from .custom_error import CustomError, IErrorStruct


class DatabaseConnectionError(CustomError):
    status_code = 500
    message = 'Error connecting to database'

    def __init__(self):
        super().__init__(self.message)

    def serialize_errors(self) -> List[IErrorStruct]:
        return [IErrorStruct(message=self.message).__dict__]



from setuptools import setup, find_packages

setup(
    name='casshoo',
    version='1.0.0',
    description='Custom errors and middlewares',
    author='yash ',
    author_email='yash.m@royalecheese.com',
    packages=find_packages(),
    install_requires=[
        'Django>=3.2',
        'confluent-kafka'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
    ],
    include_package_data=True,
)



from django.http import JsonResponse
from ..errors.custom_error import CustomError

class ErrorHandlingMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_exception(self, request, exception):
        if isinstance(exception, CustomError):
            return JsonResponse({'errors': exception.serialize_errors()}, safe=False, status=exception.status_code)
        else:
            return JsonResponse([{'message': 'something went wrong.'}], safe=False, status=400)



from .middlewares import HealthCheckMiddleware , ErrorHandlingMiddleware
from .kafka import KafkaConsumer,KafkaProducer,IConsumerConfiguration,IEvent,IMetadata,IProducerConfiguration 
from .errors import CustomError ,IErrorStruct, DatabaseConnectionError